// Black Box game
// $Id: Board.java 20 2006-09-05 21:27:58Z eric $
// Copyright 2006 Eric Smith <eric@brouhaha.com>
// See the accompanying file COPYING for license information.

// This class represents the Black Box game board.

package com.brouhaha.blackbox;

import java.awt.*;

import javax.swing.*;

class Board extends JPanel
{
    final int size;           // Size of game board (square).
    final int atoms;          // Number of atoms on the board.
    Square board [][];        // The squares that comprise the board.
    int guess_count;          // Number of active guesses the user has made.
    boolean done;             // True when the game is over.
    public Score score;       // User's score (lower is better).
    int ray_endpoint_number;  // Used to sequentially assign unique
                              // numbers to pairs of ray endpoints.

    // Allocate a pseudo-random number generator seeded from the system time.
    java.util.Random prng = new java.util.Random (System.currentTimeMillis ());

    // Construct a square Board of the default size 8x8 with four atoms.
    Board ()
    {
    	this (8, 4);
    }

    // Construct a square Board of a given size.
    Board (int size, int atoms)
    {
    	this.size = size;
    	this.atoms = atoms;
    	
    	guess_count = 0;
    	score = new Score ();

    	// Note that the following use of the Coordinate constructor
    	// initializes the size of the board.
    	new Coordinate (size, size, 0, 0);  

    	board = new Square [size + 2] [size + 2];

    	for (int x = 0; x <= size + 1; x++)
	    for (int y = 0; y <= size + 1; y++)
    		{
		    Coordinate coord = new Coordinate (x, y);
		    if (coord.corner ())
			board [x] [y] = new Corner (this, coord);
		    else if (coord.edge ())
			board [x] [y] = new Edge (this, coord);
		    else
			board [x] [y] = new Square (this, coord);
    		}

    	setLayout (new GridLayout (size + 2, size + 2));
    	for (int x = 0; x <= size + 1; x++)
	    for (int y = 0; y <= size + 1; y++)
		add (board [x] [y]);

    	reset ();  // Reset board and add atoms
    }
    
    public void reset ()
    {
    	for (int x = 0; x <= size + 1; x++)
	    for (int y=0; y <= size + 1; y++)
		board [x] [y].reset ();

    	for (int i = 0; i < atoms; i++)
    	    PlaceAtomRandomly ();
    	
    	done = false;
    	guess_count = 0;
    	score.reset ();
    	ray_endpoint_number = 0;
    	repaint ();
    }

    void PlaceAtomRandomly ()
    {
    	int x, y;

    	do
	    {
    		x = prng.nextInt (size) + 1;
    		y = prng.nextInt (size) + 1;
	    }
    	while (board [x] [y].has_atom);
    	board [x] [y].has_atom = true;
    }

    Square get_square (Coordinate coord)
    {
    	return board [coord.x] [coord.y];
    }
    
    public void submit_guesses ()
    {
    	done = true;
    	
    	// Compute and add penalty for incorrect guesses to score.
    	for (int x = 1; x <= size; x++)
	    for (int y = 1; y <= size; y++)
		if (board [x][y].has_atom && ! board [x][y].guess)
		    score.increment (5);
    	
    	score.game_over ();
    	repaint ();
    }

    // We don't need serialization, but the following avoids
    // warnings from Eclipse.
    private static final long serialVersionUID = 1L;
}
