// Black Box game
// $Id: Edge.java 23 2006-09-07 01:13:46Z eric $
// Copyright 2006 Eric Smith <eric@brouhaha.com>
// See the accompanying file COPYING for license information.

// This subclass of Square represents a square along the edge of the
// game board (but not the actual corners).  The user may click on an
// edge square to fire a Ray into the black box, and the results will
// be displayed by this edge square and possibly another.

package com.brouhaha.blackbox;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;

class Edge extends Square
{
    final Heading initial_heading;  // Initial heading for rays originating
                                    // from this edge position.

    Ray ray;  // If allocated, gives the results of a ray fired into the
              // box from this edge position.

    private Shape hit_circle;
    private Shape hit_arrow;

    private Shape refl_rect;
    private Shape refl_arc;
    private Shape refl_arrow;
    private BasicStroke refl_stroke = new BasicStroke (10.0f);

    Edge (Board board, Coordinate coord)
    {
	super (board, coord);
	if (coord.left_edge ())
	    initial_heading = Heading.east;
	else if (coord.right_edge ())
	    initial_heading = Heading.west;
	else if (coord.top_edge ())
	    initial_heading = Heading.south;
	else // if (coord.bottom_edge ())
	    initial_heading = Heading.north;

	int [] hit_arrow_xpoints = { 45, 55, 55, 65, 50, 35, 45, 45 };
	int [] hit_arrow_ypoints = { 90, 90, 55, 55, 40, 55, 55, 90 };

	hit_circle = new Ellipse2D.Double (35, 10, 30, 30);
	hit_arrow = new Polygon (hit_arrow_xpoints,
				 hit_arrow_ypoints,
				 hit_arrow_xpoints.length);

	int [] refl_arrow_xpoints = { 30, 30, 40, 60, 70, 70, 60, 75,
				      90, 80, 80, 60, 40, 20, 20, 30 };
	int [] refl_arrow_ypoints = { 90, 40, 30, 30, 40, 75, 75, 90,
				      75, 75, 40, 20, 20, 40, 90, 90 };

	refl_arrow = new Polygon (refl_arrow_xpoints,
				  refl_arrow_ypoints,
				  refl_arrow_xpoints.length);
    }
    
    void reset ()
    {
    	ray = null;
    }

    // This square is an edge, so we've exited the black box.  No
    // further ray movement is required.  Note that if the previous
    // coordinate was an edge, we haven't actually entered the box,
    // and treat the result as a reflection.
    void process_ray (Ray ray)
    {
	if (ray.current_coord.edge ())
	    ray.final_coord = ray.initial_coord;
	else
	    ray.final_coord = coord;
    }

    // This method is invoked when the user clicks on the edge, to launch
    // a ray into the black box.  The process_ray() method of Square and
    // its sublasses will be invoked recursively until the destination of
    // the ray is discovered, then this edge square and possibly one other
    // will display the results.
    void launch_ray ()
    {
	ray = new Ray (coord, initial_heading);
	next_square (ray.heading).process_ray (ray);
	
	if ((! ray.hit) && (! ray.reflected ()))
	    {
		ray.endpoint_number = ++board.ray_endpoint_number;
		Square far_endpoint = board.get_square (ray.final_coord);
		((Edge) far_endpoint ).set_ray_endpoint (coord);
	    }
	board.score.increment ();
	repaint ();
    }

    // This method is used to mark the far endpoint of a Ray.
    private void set_ray_endpoint (Coordinate other_coord)
    {
	ray = new Ray (coord, initial_heading);
	ray.final_coord = other_coord;
	ray.endpoint_number = board.ray_endpoint_number;
	board.score.increment ();
	repaint ();
    }

    // Compose a transform to make the user coordinates of this
    // JPanel ([0..100], [0..100]).
    private void setup_transform (Graphics2D g2)
    {
    	Dimension size = getSize ();

	g2.scale (size.getWidth () / 100, size.getHeight () / 100);
	g2.translate (50, 50);
	g2.rotate (Math.toRadians (270 - initial_heading.angle));
	g2.translate (-50, -50);
    }

    private void show_hit (Graphics2D g2)
    {
	setup_transform (g2);

	// Draw green circle representing atom.
	g2.setPaint (Color.green);
	g2.fill (hit_circle);

	// Draw blue arrow representing ray.
	g2.setPaint (Color.blue);
	g2.fill (hit_arrow);
    }

    private void show_reflection (Graphics2D g2)
    {
	setup_transform (g2);

	// Draw blue u-turn arrow representing ray.
	g2.setPaint (Color.blue);
	g2.fill (refl_arrow);
    }

    public void paintComponent (Graphics g)
    {
	Graphics2D g2 = (Graphics2D) g;
	String s;

	draw_outline_and_background (g2, Color.gray, Color.white);

	if (ray == null)
	    return;

	if (ray.hit)
	    show_hit (g2);
	else if (ray.reflected ())
	    show_reflection (g2);
	else
	    {
		s = Integer.toString (ray.endpoint_number);
		g2.setPaint (Color.black);
		g2.drawString (s, 5, 15);
	    }
    }

    // If the game is in progress, and there hasn't already been a ray
    // launched from (or terminating at) this square, launch a new ray.
    public void mouseClicked (MouseEvent event)
    {
	if (! board.done && (ray == null))
	    launch_ray ();
    }

    // We don't need serialization, but the following avoids
    // warnings from Eclipse.
    private static final long serialVersionUID = 1L;
}
