// Black Box game
// $Id: Square.java 16 2006-09-03 07:55:13Z eric $
// Copyright 2006 Eric Smith <eric@brouhaha.com>
// See the accompanying file COPYING for license information.

// This class represents a square within the body of the
// BlackBox game board, and is also the base class from which the
// Edge and Corner classes are derived.

// Each square has a reference to the board it belongs to, as we

// The square has a flag to record whether an atom is present, and
// another to record whether the user guesses that an atom may be
// present.  Clicking the mouse on a square toggles the guess flag.
// The guess is visually represented by a blue circle.

// A method is provided to return a reference to an adjacent (next)
// square given a heading.

// The process_ray() method is the core of the game logic.  It determines
// whether the ray scores a hit, enters this square, or is deflected
// *prior* to entering.

package com.brouhaha.blackbox;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import javax.swing.*;

class Square extends JPanel
    implements MouseListener
{
    protected Board board;  // Reference to the board that we're part of.
    protected Coordinate coord;   // Coordinate of this Square within board.

    boolean has_atom;  // True if this square contains an atom.
    boolean guess;  // True if the user has guessed that an atom might be here.

    Square (Board board, Coordinate coord)
    {
    	this.board = board;
    	this.coord = coord;
    	addMouseListener (this);
    	reset ();
    }
    
    void reset ()
    {
    	has_atom = false;
    	guess = false;
    }

    // Return the next square on the board in a given heading.
    Square next_square (Heading heading)
    {
	return (board.get_square (coord.next_coord (heading)));
    }

    // This method is used to deal with a Ray that is about to enter this
    // square.  Note that when this method is called, the current location
    // of the Ray is an adjacent square, not this one.  If there is an
    // atom in this square, the ray scores a hit.  Otherwise, depending on
    // whether there are atoms in certain nearby squares, the ray may be
    // deflected *instead* of entering this square.
    void process_ray (Ray ray)
    {
	if (has_atom)
	    {
		ray.hit = true;
	    }
	else if (next_square (ray.heading.counterclockwise ()).has_atom)
	    {
		ray.heading = ray.heading.clockwise ();
		board.get_square (ray.current_coord).process_ray (ray);
	    }
	else if (next_square (ray.heading.clockwise ()).has_atom)
	    {
		ray.heading = ray.heading.counterclockwise ();
		board.get_square (ray.current_coord).process_ray (ray);
	    }
	else
	    {
		ray.current_coord = coord;
		next_square (ray.heading).process_ray (ray);
	    }
    }

    protected final static Color buff  = new Color (255, 255, 191);

    protected final static BasicStroke wideStroke = new BasicStroke (4.0f);

    private void draw_circle (Graphics2D g2, Color color)
    {
    	Dimension size = getSize ();
    	g2.setPaint (color);
	g2.fill (new Ellipse2D.Double (5, 5,
				       size.width - 10, size.height - 10));
    }
    
    private void draw_x (Graphics2D g2, Color color)
    {
       	Dimension size = getSize ();
	g2.setPaint (color);
	g2.setStroke (wideStroke);
	g2.draw (new Line2D.Double (5, 5,
				    size.width - 10, size.height - 10));
	g2.draw (new Line2D.Double (5, size.height - 10,
				    size.width - 10, 5));
    }
    
    protected void draw_outline_and_background (Graphics2D g2,
						Color stroke_color,
						Color fill_color)
    {
    	Dimension size = getSize ();
	Rectangle r = new Rectangle (0, 0,
				     size.width,
				     size.height);
	g2.setPaint (fill_color);
	g2.fill (r);
	if (stroke_color != null)
	    {
		g2.setPaint (stroke_color);
		g2.draw (r);
	    }
    }
    
    // Display the square.  During game play, only guesses are shown, as
    // blue circles.  If the game is over, a guess in this square must be
    // wrong, since there is no atom here, so it will be displayed as a
    // blue circle with a red X through it.
    public void paintComponent (Graphics g)
    {
    	Graphics2D g2 = (Graphics2D) g;
    	
    	draw_outline_and_background (g2, Color.black, buff);

    	if (! board.done)
	    {
		if (guess)
		    draw_circle (g2, Color.blue);
	    }
    	else
	    {
		if (has_atom && guess)
		    draw_circle (g2, Color.green);
		else if (has_atom)
		    draw_circle (g2, Color.red);
		else if (guess)
		    {
			draw_circle (g2, Color.blue);
			draw_x (g2, Color.red);
		    }	
	    }
    }
 
    public void mousePressed  (MouseEvent event) {}
    public void mouseReleased (MouseEvent event) {}
    public void mouseEntered  (MouseEvent event) {}
    public void mouseExited   (MouseEvent event) {}

    // Upon a mouse click while the game is in progress, toggle the
    // user's guess.
    public void mouseClicked (MouseEvent event)
    {
    	if (board.done)
	    return;
    	if (guess)
    	{
	    guess = false;
	    board.guess_count--;
    	}
    	else
    	{
	    if (board.guess_count >= board.atoms)
		return;
	    guess = true;
	    board.guess_count++;
    	}
    	repaint ();
    }

    // We don't need serialization, but the following avoids
    // warnings from Eclipse.
    private static final long serialVersionUID = 1L;
}
